/* --- Copyright University of Birmingham 1999. All rights reserved. ------
 > File:            $poplocal/local/auto/sys_host_type.p
 > Purpose:			Get information about current terminal
 > Author:          Aaron Sloman and Andy Williams, Oct 28 1992 (see revisions)
 > Documentation:	Below
 > Related Files:
 */

/*
sys_host_type(hostname) -> string
	Given alphanumeric host name returns a string describing the machine type.
	Given numeric internet address ditto

*/

section;


lconstant
	dig_command = sysfileok('/bham/bin/cpu');

lconstant
	arglist = [^dig_command 0];

lvars answer_found = false;

define lconstant procedure get_host_data(hostname) -> string;
	;;; if it's alphabetic, get the host type. If it's numeric
	;;; get the name string.
	lvars
		hostname, field_spec,
		pipe_device, repeater,
		string, loc, fin;

	define lconstant cleanup();
		sysclose(pipe_device);
		if pop_internal_version < 150000 then
			syswait() -> ;		;;; leave no zombies	
		endif;
	enddefine;

	define timeout();
		if answer_found then
			false -> answer_found
		else
			;;; sys_wait(false) -> (,);
			termin;
			exitfrom(repeater)
		endif;
	enddefine;
		

	if sys_file_exists(dig_command) then
		;;; create arglist for dig_command
		hostname -> arglist(2);

		;;; spawn dig process, and get pipe device to read in output
		pipein(dig_command, arglist, false) -> pipe_device;

		;;; make a string repeater from the device
		line_repeater(pipe_device, sysstring) -> repeater;

		;;; get output line containing field_spec
		repeat
			false -> answer_found;
			;;; Allow at most three seconds wait
			3e6 -> sys_timer(timeout);
			repeater() -> string;
			true -> answer_found;
			if string == termin then
				;;; field line not found
				cleanup();
				false -> string;
				return()
			endif;
		quitunless(datalength(string) == 0);
		endrepeat;

		cleanup();
	else
		false -> string;	;;; Let user's default decide
	endif
enddefine;

/*
define global procedure sys_host_label =
	get_host_data(%false%)
enddefine;
*/

define global procedure sys_host_type(hostname) -> string;
	lvars hostname, string;
	if hostname then
		get_host_data(hostname)
	else false
	endif -> string;

	unless string then
		'Terminal type not recognised'=>

		'Please specify type by typing one letter'=>
		'S(Sun), N(Ncd), P(PC), E(Exceed on PC) A(Alpha), H(HP)' =>
		lvars char = charin();
		if char == `s` or char == `S` then 'Sun'
		elseif char == `e` or char == `e` then 'exceed'
		elseif char == `n` or char == `N` then 'NCD'
		elseif char == `p` or char == `P` or char == `h` or char == `H`
		or char == `n` or char == `N` then 'NCD'
		else false
		endif -> string
	endunless;
enddefine;

endsection;

/* --- Revision History ---------------------------------------------------
--- Aaron Sloman, Sep 25 1999
		Extended to cope with exceed
--- Aaron Sloman,  8 Sep 1996
	Patch for Alphas and machines without the 'cpu' command
--- Aaron Sloman, Nov 21 1995
	Put a test on syswait, for V15
 */
