/* --- Copyright University of Birmingham 2002. All rights reserved. ------
 > File:            $poplocal/local/auto/xplatform.p
 > Purpose:         Get information about current terminal
 > Author:          Aaron Sloman and Andy Williams, Oct 28 1992 (see revisions)
 > Documentation:   Below
 > Related Files:
 */

/*
xplatform() -> string
Returns type of host, as obtained using sys_host_type with host
name or address in $DISPLAY variable.

*/
section;

define lconstant procedure cut_to(char, string);
	lvars char, string, n = strmember(char, string);
	if n then
		substring(1, n-1, string)
	else
		mishap(0, string sys_>< ' DOES NOT CONTAIN ' sys_>< consstring(char,1))
	endif
enddefine;


define global constant procedure xplatform() -> string;

	lvars
		display = systranslate('DISPLAY'),
		machine = hd(sys_machine_type),
		terminal_host;

	if isstring(display) then
		if isstartstring('psg', display) then
			;;; assume PC keyboard in psychology is like NCD. May need to change it
			'NCD' -> string
		elseif isstartstring('acws', display) then
			;;; assume it is a PC in the School of Computer Science
			'NCD' -> string
		elseif display = ':0.0' or display = ':0' then
			;;; Running locally on workstation
			if machine == "alpha" then
				'DEC-alpha'
			elseif isstartstring('sun', machine) then
				'Sun'
			elseif machine == "pc" then
				;;; assume PC keyboard is like NCD. May need to change it
				'NCD'
			else
				sys_host_type(sys_host_name())
			endif -> string;
		elseif display then
			cut_to(`:`, display) -> terminal_host;
			if isstartstring('ncd', terminal_host) then
				'NCD'
			else
				sys_host_type(terminal_host)
			endif -> string;
		endif;
	else sys_host_type(false) -> string
	endif;
enddefine;

endsection;

/* --- Revision History ---------------------------------------------------
--- Aaron Sloman, Feb 18 2002
		Changed to detect new linux PCs
--- Aaron Sloman, Feb 11 2000
	Generalisation to cope with '.0' thanks to Andrew Sayers.
--- Aaron Sloman, Oct 28 1997
	Changed to recognize psychology machines
--- Aaron Sloman,  8 Sep 1996
	Generalised for Alphas
--- Aaron Sloman, Feb  8 1995
	Because the 'cpu' command does not work (yet) on Solaris, this has
	been changed to use a hack to detect sun4 keyboards by recognising
	that it is running on solaris with DISPLAY = ':0.0'
 */
