/* --- Copyright University of Sussex 1992. All rights reserved. ----------
 > File:            $poplocal/local/lib/arrayfile.p
 > Purpose:         Reading and writing bit-packed arrays to/from disk
 > Author:          David Hogg, January 1989 (see revisions)
 > Documentation:   HELP *ARRAYFILE
 */

section;

define stringout(s) /* -> proc */;
    lvars s;
    lvars i = 1;

    procedure(byte);
        lvars byte;
        byte -> s(i);
        i + 1 -> i
    endprocedure
enddefine;

lconstant infoblk = inits(512);

define arrayfile(filename) -> array;
    lvars filename, array = false;
    lvars device, numread, bounds, buffer, key, type, min_sub;
    ;;; allow optional second (array) argument so that it can read into
    ;;; an existing array
    if filename.isarray then filename -> (filename, array) endif;
    sysopen(filename,0,true) -> device;
    unless device.isdevice then
        mishap('Specified file does not exist',[^filename]);
    endunless;
    erase(sysread(device,infoblk,512));
    incharitem(stringin(infoblk)) -> numread;
    [% repeat 2*numread() times numread() endrepeat %] -> bounds;
    numread() -> type;
    if type = termin then "string" -> type endif;
    key_of_dataword(type) -> key;
    unless key then mishap(key, 1, 'Unknown vectorclass') endunless;
    if array then
        unless boundslist(array) = bounds and
            datakey(arrayvector(array)) == key then
            mishap(filename, array, 2, 'Array not compatible with file data')
        endunless;
        arrayvector_bounds(array) -> min_sub -> ;
        unless min_sub == 1 then
            mishap(array, 1, 'Array must not be offset in arrayvector')
        endunless
    else
        newanyarray(bounds,key) -> array
    endif;

    arrayvector(array) -> buffer;
    erase(sysread(device,buffer,4*(datasize(buffer)-2)));
    sysclose(device);
enddefine;

define updaterof arrayfile(array,filename);
    lvars array, filename;
    lvars device, buffer, infostring;
    dlocal cucharout = stringout(infoblk);
    stringout(infoblk) -> cucharout;
    spr(intof(length(boundslist(array))/2));
    applist(boundslist(array),spr);
    arrayvector(array) -> buffer;
    spr(dataword(buffer));
    cucharout(0);
    syscreate(filename,1,true) -> device;
    syswrite(device,infoblk,512);
    syswrite(device,buffer,4*(datasize(buffer)-2));
    sysclose(device);
enddefine;

endsection;

/* --- Revision History ---------------------------------------------------
--- David S Young, Jan  3 1992 A file can now be read into an existing array
--- David S Young, Aug  2 1991
        infoblk made file lconstant (was lvar in each procedure)
        cucharout made dlocal (was vars)
        stringout changed to use lvar instead of ref to hold index,
            closure now implicit rather than explicit
        arguments made lvars
        section ... endsection added
--- David Young, Feb 12 1990 Changed header comment format so that
    newmaster transport can work
 */

