/* --- Copyright University of Birmingham 1998. All rights reserved. ------
 > File:            $poplocal/local/lib/buildindex.p
 > Purpose:         See below
 > Author:          Aaron Sloman, Jul  7 1997 (see revisions)
 > Documentation:
 > Related Files:
 */

/* --- Copyright University of Sussex 1993. All rights reserved. ----------
 > File:            C.all/lib/lib/buildindex.p
 > Purpose:			Create index files for use with popindex and ved_sourcefile
 > Author:          Aaron Sloman, Apr 10 1989 (see revisions)
 > Documentation:	HELP POPINDEX
 > Related Files:	LIB * POPINDEX, * VED_SOURCEFILE, $POPCOM/MKIND*
 */

#_TERMIN_IF DEF POPC_COMPILING

/*

Run this as
	buildindex(popindexlist, popindex_filename, boolean);

where
	popindexlist is a list of patterns to match to get files to analyse,
	popindex_filename is a file name to which suffixes indicating contents
		should be added for the different index entries
	boolean is true if logical names (e.g. $usepop, $popvedlib) should
		be preserved at the front of file names in index entries.



Currently uses -popindex_suffixlist- and -choose-index-, as defined
in LIB * POPINDEX

E.g. popindexlist might be defined as
(Spaces in '/ *' below to defeat Pop-11 comment reader)

	[%  '$popsrc/ *.p',
		for item in popuseslist do
			item dir_>< '*.p';
		endfor,
		'$usepop/pop/plog/src/ *.p',
		'$usepop/pop/plog/lib/ *.p',
		'$usepop/pop/plog/auto/ *.p',
		'$usepop/pop/lib/flavours/ *.p'
		%] -> popindexlist;


Note that the actual value for popindexlist used by default isdefined in
	$popcom/mkind		(Unix)
	$popcom:mkind.com	(VMS)


popindex_filename might be defined as

	'$usepop/pop/ref/popindex.' -> popindex_filename

*/

section;
true -> popconstruct;	;;; for pattern lists

uses popindex;	;;; contains global definitions e.g. choose_index
				;;; and popindex_suffixlist


;;; Next two partly adapted from ved_f

;;; predicate for finding name
define constant procedure terminator =
	lmember(% [% "(", ";", "=>", "->", "with_nargs", "with_props", "=" %] %)
enddefine;

define constant procedure isspecial =
	lmember(%[define updaterof vars constant lconstant global macro syntax
			iconstant
			active : nonactive dlocal lvars procedure ^("%", "'/*'", "'*/'" ) ] %)
enddefine;


define lconstant splitstring(string) -> list;
	;;; Parse string after 'define '
	lvars procedure rep, item, string, list;
	incharitem(stringin(string)) -> rep;
	[% until (rep() ->> item) == termin do item enduntil %] -> list;
enddefine;


vars xx, yy;	;;; pattern variables

define lconstant find_name(def_line) -> name;
	;;; given a procedure header string, find the name
	lvars  next, isupdater = false, name;
	dlocal xx, yy;		;;; pattern variables
	splitstring(def_line) -> def_line;
	if def_line == [] then
		false -> name;
	else
		;;; now cope with 'define :<form> ...'
		if fast_front(def_line) == ":" then
			fast_back(fast_back(def_line)) -> def_line
		endif;
		repeat
			if def_line == [] then
				'A_NO_NAME' -> name; return
			endif;
			fast_front(def_line) -> next;
			if next == "updaterof" then
				true -> isupdater;
				fast_back(def_line) -> def_line;
				nextloop();
			endif;
		quitunless(isspecial(next));
			fast_back(def_line) -> def_line;
			if next == "active" then
				if def_line == [] then
				elseif fast_front(def_line) == ":" then
					;;; should check for colon then integer
					back(fast_back(def_line)) -> def_line;
				endif
			endif
		endrepeat;
		;;; get stuff before terminator, include "("
		while def_line matches [??xx ?yy:terminator ==] do
			;;; get rid of stuff after terminator
			xx -> def_line
		endwhile;
		if  atom(def_line) then
			false -> name
		elseif isnumber(fast_front(def_line)) then
			fast_back(def_line) -> def_line;
			if atom(def_line) then
				false -> name;
			elseif def_line matches [ = ( ==] then
				fast_front(def_line) -> name;
			elseif listlength(def_line) == 3 then
				def_line(2) -> name;
			else
				def_line(1) -> name;
			endif;
			;;;; THAT DOES NOT COPE WITH SECTION RELATIVE IDENTIFIERS
		else
			;;; get rid of section path names
			if fast_front(def_line) == "$-" then
				fast_back(def_line) -> def_line;
				while def_line matches [= $- ==] do
					def_line.fast_back.fast_back -> def_line
				endwhile;
			else
				while def_line matches [= $- ??xx] do
					xx -> def_line
				endwhile;
			endif;
			if def_line == [] then false
			else
				fast_front(def_line)
			endif -> name
		endif
	endif;
	if isupdater and name then name >< ' updaterof' -> name endif;
enddefine;


define lconstant make_entry(dir_pattern, file_name, entryprop, preserve_environ);
	;;; Get procedure definitions from the file and store information about
	;;; them in the appropriate index list in entryprop
	;;; Use dir_pattern to truncate the path name using logical names
	lconstant
		;;; root directory for poplog
		usepopstring = '$usepop',
		;;; length of expanded version
		usepoplength = datalength(sysfileok(usepopstring)),
		buffer = inits(512),
	;

	lvars fd, _len, dir_pattern, full_dir,
		file_name, logical, preserve_environ, procedure entryprop;

	dlocal popfilename = file_name, poplinenum = 1;	;;; for debugging

	define dlocal prmishap() with_nargs 2;
		warning();
		exitfrom(make_entry)
	enddefine;

	sysopen(file_name, 0, "record", `N`) -> fd;

	if preserve_environ and isstartstring(usepopstring, dir_pattern) then
		;;; Remove actual path name. Put back logical one
		usepopstring dir_>< allbutfirst(usepoplength, file_name)
	else
		if preserve_environ
		and (locchar(`$`,1,dir_pattern) or locchar(`~`,1,dir_pattern)) then
			;;; handle other logical names and (on Unix) user names.
			if (locchar(`/`,2,dir_pattern) ->> _len) then
				;;; get logical name starting dir_pattern
				substring(1, _len, dir_pattern) -> logical;
				sysfileok(logical) -> full_dir;
				logical dir_>< allbutfirst(datalength(full_dir), file_name)
			else goto USE_FILE_NAME
			endif
		else
			USE_FILE_NAME:
			sys_fname(file_name, 1,5)	;;; strips version
		endif
	endif -> file_name;

	until (fast_sysread(fd, 1, buffer, 512) ->> _len) == 0 do
		if  (fast_subscrs(1, buffer) == `d`)
		and isstartstring('define', buffer)
		and _len /== 6
		;;; deal with 'define ' and 'define:'
		and strmember(fast_subscrs(7, buffer), '\s:')
		then
			lvars p_name = find_name(substring(7, _len fi_- 6, buffer));
			if p_name then
				lvars index_name = consword(choose_index(p_name));
				lvars
					entry = p_name sys_>< ' ' <> file_name,
					entries = entryprop(index_name);
				unless member(entry, entries) then
					conspair(entry, entries) -> entryprop(index_name);
				endunless;
			endif
		endif;
		poplinenum + 1 -> poplinenum
	enduntil;

	sysclose(fd);
enddefine;

define lconstant get_entries(index_list, entryprop, preserve_environ);
	lvars procedure (file_rep, entryprop), dir, file_name, preserve_environ,
		index_list, index_filename;

	dlocal
		% item_chartype(`*`) % = 3,
		% item_chartype(`/`) % = 3,
		% item_chartype(`'`) % = 1,
		;

	fast_for dir in index_list do
		;;; semi-colon in second argument needed for VMS
		sys_file_match( dir, '*.p;', false, false) -> file_rep;
		until (file_rep() ->> file_name) == termin do
			unless issymlink(file_name) then
				make_entry(dir, file_name, entryprop, preserve_environ);
			endunless;
		enduntil;
	endfor;
enddefine;

define lconstant sort_and_write(suffixlist, entryprop, fileprop);
	lvars output_device, entry, i_name, list, suffix, suffixlist,
		procedure (entryprop, fileprop);

	fast_for suffix in suffixlist do
		consword(suffix) -> i_name;
		syssort(entryprop(i_name), alphabefore) -> list;
		fileprop(i_name) -> output_device;
		fast_for entry in list do
			fast_syswrite(output_device, 1, entry, datalength(entry));
			fast_syswrite(output_device, 1, '\n', 1);
		endfor;
		sysclose(output_device);
	endfor;
enddefine;

define lconstant prune_list(list)-> newlist;
	lvars item, newlist=[];
	for item in list do
		unless member(item, newlist) then
			conspair(item, newlist) -> newlist
		endunless;
	endfor;
	fast_ncrev(newlist) -> newlist;
enddefine;

define buildindex(index_list, index_filename, preserve_environ);
	lvars index_list, index_filename, suffix, i_name, preserve_environ;
	lconstant procedure(
		list_of_entries = newproperty([], 100, [], true),
		index_file = newproperty([], 100, false, true)
	);
	;;; get rid of repetitions in the list
	prune_list(index_list) -> index_list;
	;;; create the output devices and initialise lists for sorting
	for suffix in popindex_suffixlist do
		consword(suffix) -> i_name;
		unless (syscreate(
					sysfileok(index_filename sys_>< suffix), 1, false)
					->> index_file(i_name)) then
			printf(suffix, '\ncannot create index file popindex.%p\n');
			sysexit();
		endunless;
	endfor;

	[get the entries] =>
	get_entries(index_list, list_of_entries, preserve_environ);
	[sort the lists] =>
	sort_and_write(popindex_suffixlist, list_of_entries, index_file);
	[all done] =>
enddefine

endsection;


/*  --- Revision History ---------------------------------------------------
--- Aaron Sloman, Feb 22 1998
		Changed to ignore symbolic links, which generally duplicate another
		entry.
--- Aaron Sloman, Jul  7 1997
		Made to prune the list, and also avoid duplicate entries in
		popindex file
		Put iconstant in the special list
--- John Williams, Apr 30 1993
		___________get_entries now dlocal's the item_chartype of / * and '
		(instead of changing them globally)
--- John Williams, Apr 30 1993
		__________make_entry now dlocal's prmishap to warning
--- John Williams, Apr 30 1993
		__________make_entry now dlocal's popfilename and poplinenum, in case errors
		happen while parsing procedure definitions
--- Aaron Sloman, Jul 30 1990
		Slightly altered comments
--- Aaron Sloman, May 20 1990
		Altered to allow define:form... as well as define :form...
--- John Gibson, Aug  4 1989
		Replaced use of -sysfiledir- and -sysfullfilename- with -sys_fname-
--- Aaron Sloman, May  1 1989 removed uses of pdr_valof
--- Aaron Sloman, Apr 22 1989
	Changed to allow define ... foo = ....; enddefine;

	Also gave -buildindex- (and other procedures) an extra argument to
	specify whether environment variables (e.g. $usepop) should be preserved
	at the beginning of file names in index entries. (Saves space and can
	make the index files more portable)

--- Aaron Sloman, Apr 11 1989
	Fixed to cope with active identifiers and their updaters
--- Aaron Sloman, Apr 10 1989
	Much modularised. Created top level procedure buildindex, invoked
	from $popcom/mkind file
*/
