/* --- The University of Birmingham 1994.  --------------------------------
 > File:			$poplocal/local/lib/connect4.p
 > Purpose:			Library program to play connect4
 > Authors:			See below (MSc Group "mini project" Dec 1993)
 >					Installed by Aaron Sloman, Jul 12 1994
 >					With some slight edits for smooth startup
 > Documentation:	$poplocal/local/doc/connect4.ps (postscript file)
 > Related Files:	HELP * CONNECT4, LIB * GO
 */

;;; Modified by A.Sloman for Poplog V15. 26 Aug 1997

/*
   ********************************************************************
   ********************************************************************
   ***																***
   ***							Connect 4							***
   ***	 By	  Oliver Harvey , Simon Goodwin	 and  Ram Herkanaidu	***
   ***																***
   ***						   17/12/93								***
   ********************************************************************
   ********************************************************************
*/

;;; ------- to start up connect4 you will require access to :-

;;;					 (1) lib go, or go saved image (pop11 +go)
;;;					 (2) lib rc_mouse	 - libraries.
;;;					 (3) lib connect4  - this file.

;;; ------- Loading this file in pop11 will take a while as 'Objectclass'
;;; ------- and GO must be loaded, and connect4 compiled.

;;; When you have connect4 loaded type, to Pop-11
;;;		start_up();

vars colour gamestate column boxhighlightcol;

uses go;
uses rc_mouse;

;;; declare game constants.
global constant max_columns=7, max_rows=6;

;;; And all the procedure names.
;;; Declare them as procedure vars for reduced run-time type checking.
;;; rearranged to alphabetical order by A.S.
global vars
	procedure
		(bookmove, calc_moves, calc_gamestate, change_skill,
		check_filed, check_legal, check_piece, clear, comp_move,
		consave, do_backwards, do_draw, do_end, do_hint, do_load,
		do_quit, do_reset, do_rules, do_save, do_win, draw_new_state,
		empty, evaluate, evaluate_position, finish,
		gamestate_from_history, generate_alternatives, get_input,
		grid, input_loop, load_game, make_move, minimax, objects,
		opposite, option_check, play, player_move, plot_gamestate,
		remove_counter, set_up_boxes, setup, show_message, show_whose_move,
		start, start_playervcomp, test_boxes, toggle,
		update_calc_and_redraw, update_history, update_max_depth_level,
		update_mode, update_moves, update_skill, window_up),
	reset;


;;; set-up board and constants. -
;;; These constants must be set so that the class instances can have their
;;; parameters changed.

define start_up();
	vars info_box_moves_so_far, info_box_skill, moves, max_depth_level,
		info_box_mode, text_input_box, message_box, forwards, backwards,
		reset, text_input_box, empty, max_depth_level,
		mode, outputstring, button_pos, pane1, skill_level,
		bookmove_flag = false,
		;

	0 -> moves;
	'Intermediate' -> skill_level;

	setup() -> (message_box, info_box_moves_so_far, info_box_skill,
		info_box_mode, text_input_box, forwards, backwards,
		reset, text_input_box);

	show_message(['Welcome to connect 4' 'These are the info  boxes'
			' above are the options' 'starting with two players']);

	start('two_players');
enddefine;

;;; calls the minimax procedure , counts time , updates the skill level
;;; and returns the column chosen.
define make_move(position, player) -> index;
	 vars  result, move, best_result;
	 lvars position, player, index, time_elapsed;

	 update_max_depth_level( skill_level) -> max_depth_level;

	 if moves = 10 then max_depth_level + 1 -> max_depth_level; endif;
	 if moves = 15 then max_depth_level +1 -> max_depth_level; endif;

	 sys_real_time() -> time_elapsed;
	 minimax(position, 0, player, -2000, 2000) -> result;
	 result --> [?move ?best_result];
	 1 -> index;
	 repeat
		 quitif(index == max_columns or (position(index) /== move(index)));
		 index + 1 -> index;
	 endrepeat;
	 sys_real_time() - time_elapsed -> time_elapsed;
		 ;;;  [total time taken is ^time_elapsed seconds]
enddefine;

define update_max_depth_level(skill_level);
	switchon skill_level =
	case 'Beginner' then return(1);
	case 'Intermediate' then return(2);
	case 'Expert' then return(3);
	endswitchon;
enddefine;

;;;
;;; Procedure Evaluate_Position examines a given list eg. [0 1 -1 0 1 1 0],
;;; 0 for a blank piece, 1 for current player and -1 for the opponent,
;;; looking for as many of the possible strategy combinations within it. If
;;; one is found then associated weighted value is added to a running total.
;;; The final total is returned.
;;;

define evaluate_position(pieces) -> result;
	vars position_strategies, strategy, pieces, value, position;
	lvars result;
	0 -> result;
	[[[1 0 0 1] 3] [[1 0 1 0] 3] [[0 1 0 1] 3] [[0 1 0 1 0] 4] [[1 1 0 0] 5]
	 [[0 0 1 1] 6] [[0 1 1 0] 7] [[1 1 0 1] 8] [[1 0 1 1] 9] [[1 1 1 0] 20]
	 [[0 1 1 1] 20] [[0 1 1 1 0] 50] [[1 1 1 1] 750]] -> position_strategies;

	for strategy in position_strategies do
		strategy --> [?position ?value];
		if pieces matches [== ^^position ==] then
			result + value -> result;
		endif;
	endfor;
enddefine;

;;;
;;; Procedure check_piece examines the current piece and determines whether it
;;; is the current players, a blank or the opponents.
;;;

define check_piece(piece, player, piece_list) -> result;
	lvars piece, player, piece_list, result;
	if piece == player then
		[^^piece_list 1] -> result;
	elseif piece == "B" then
		[^^piece_list 0] -> result;
	else
		[^^piece_list -1] -> result;
	endif;
enddefine;

;;;
;;; Procedure Evaluation examines the current board position and determines
;;; how good that position is for a particular player. Using a set of optimal
;;; strategy moves, a value for that position is given.
;;;

define evaluate(position, player) -> result;
	lvars
		position, player, column, piece, piece_list, index, row, vertical,
		start_row, count, start_vertical, result;

	lvars procedure(board);
	;;;
	;;; generate vector table array of board position
	;;;
	newarray([1 ^max_rows 1 ^max_columns], "B") -> board;
	0 -> result;
	1 -> vertical;
	for column in position do
		1 -> row;
		for piece in column do
			piece -> board(row, vertical);
			row + 1 -> row;
		endfor;
		vertical + 1 -> vertical;
	endfor;
	;;;
	;;; check vertical pieces
	;;;
	1 -> vertical;
	repeat max_columns times
		1 -> row;
		[] -> piece_list;
		repeat max_rows times
			check_piece(board(row, vertical), player, piece_list)
				-> piece_list;
			row + 1 -> row;
		endrepeat;
		vertical + 1 -> vertical;
		if not(empty(piece_list)) then
			result + evaluate_position(piece_list) -> result;
		endif;
	endrepeat;
	;;;
	;;; check horizontal pieces
	;;;
	1 -> row;
	repeat max_rows times
		1 -> vertical;
		[] -> piece_list;
		repeat max_columns times
			check_piece(board(row, vertical), player, piece_list)
				-> piece_list;
			vertical + 1 -> vertical;
		endrepeat;
		row + 1 -> row;
		if not(empty(piece_list))  then
			result + 2*(evaluate_position(piece_list)) -> result;
		endif;
	endrepeat;
	;;;
	;;; check the left to right diagonnal
	;;;
	max_rows - 3 -> start_row;
	1 -> start_vertical;
	repeat
		[] -> piece_list;
		start_row -> row;
		start_vertical -> vertical;
		repeat
			check_piece(board(row, vertical), player, piece_list)
				-> piece_list;
			quitif(row == max_rows or vertical == max_columns);
			row + 1 -> row;
			vertical + 1 -> vertical;
		endrepeat;
		if start_row > 1 then
			start_row - 1 -> start_row;
		else
			start_vertical + 1 -> start_vertical;
		endif;
		if not(empty(piece_list)) then
			result + 2*(evaluate_position(piece_list)) -> result;
		endif;
		quitif(start_vertical == (max_columns - 2));
	endrepeat;
	;;;
	;;; check the right to left diagonnal
	;;;
	max_rows - 3 -> start_row;
	max_columns -> start_vertical;
	repeat
		[] -> piece_list;
		start_row -> row;
		start_vertical -> vertical;
		repeat
			check_piece(board(row, vertical), player, piece_list)
				-> piece_list;
			quitif(row == max_rows or vertical == 1);
			row + 1 -> row;
			vertical - 1 -> vertical;
		endrepeat;
		if start_row > 1 then
			start_row - 1 -> start_row;
		else
			start_vertical - 1 -> start_vertical;
		endif;
		if not(empty(piece_list)) then
			result + 2*(evaluate_position(piece_list)) -> result;
		endif;
		quitif(start_vertical == max_columns - 4);
	endrepeat;
enddefine;

;;;
;;; Procedure Empty determines whether for a given list of numbers
;;; eg [0 1 0 0 0 0] it contains a players piece, either 1 or -1.
;;; if no pieces exist then it returns true, ie. empty.
;;;

define empty(array);
	lvars array, result, element;
	for element in array do
		if element /== 0 then
			return(false);
		endif;
	endfor;
	return(true);
enddefine;

;;;
;;; Procedure generate_alternatives determines all the possible moves a given
;;; player could make for a given board position;
;;;

define generate_alternatives(position, player) -> successors;
	lvars position, player, successors, temp_columns, column, index;
	lvars move_position, count, alternatives;
	;;;
	;;; generate players next move for each allowable column, ie. if the
	;;; column is full then obviously a move cannot be made there.
	;;;
	[] -> temp_columns;
	for column in position do
		if length(column) < max_rows then
			[^^temp_columns [^^column ^player]] -> temp_columns;
		else
			[^^temp_columns []] -> temp_columns;
		endif;
	endfor;
	;;;
	;;; generate all possible board positions for each of the possible moves,
	;;; ie. if 6 possible moves can be made, then 6 different board positions
	;;;		are generated for each one of those moves.
	;;;
	1 -> index;
	[] -> successors;
	repeat max_columns times
		1 -> count;
		[] -> alternatives;

		if temp_columns(index) /= [] then
			for column in position do
				if count == index then
					temp_columns(index) -> move_position;
					[^^alternatives ^move_position] -> alternatives;
				else
					[^^alternatives ^column] -> alternatives;
				endif;
				count + 1 -> count;
			endfor;
			[^^successors ^alternatives] -> successors;
		endif;

		index + 1 -> index;
	endrepeat;
enddefine;

;;;
;;; Procedure opposite returns the opposite player to the one given.
;;; It is needed when determining opposite opponents moves.
;;;

define opposite(player);
	lvars player;
	if player == "Y" then
		return("R");
	else
		return("Y");
	endif;
enddefine;

;;;
;;; Procedure finish determines whether the maximum depth level has been
;;; reached or one side has won.
;;;

define finish(position, player, depth);
	lvars depth, position;
	if depth == max_depth_level then
		return(true);
	elseif evaluate(position, player) >= 750 then
		return(true);
	else
		return(false);
	endif;
enddefine;

;;;
;;; Procedure minimax determines the successive moves from a given position,
;;; using alpha - beta pruning to trim the tree structure of the search and
;;; returning the best move that should be made.
;;;

define minimax(position, depth, player, alpha, beta);
	lvars move, result, best_score, best_path, position, depth,
		player, successors;
	vars successor_result, successor_value, successor_path;
	lvars red, yellow;
	if finish(position, opposite(player), depth) then
		evaluate(position, opposite(player)) -> red;
		;;; if a win has been found, then there is no need to evaluate
		;;; the value of the opponents move.
		if red < 750 then
			evaluate(position, player) -> yellow;
		else
			0 -> yellow;
		endif;
		yellow - red -> result;
		return([^position ^result]);
	else
		;;; generate alternative moves from current positions
		generate_alternatives(position, player) -> successors;
		;;; if successors is empty then no more moves can be made
		if successors == [] then
			evaluate(position, opposite(player)) -> red;
			if red < 750 then
				evaluate(position, player) -> yellow;
			else
				0 -> yellow;
			endif;
			yellow - red -> result;
			return([^position ^result]);
		else
			;;; examine each element in turn and keep a record of the best
			;;; score. set best_score to minimum value.
			position -> best_path;
			alpha -> best_score;
			;;; find the best possible moves to examine
			for move in successors do
				minimax(move, depth + 1, opposite(player), -beta, -best_score)
					-> successor_result;
				successor_result --> [?successor_path ?successor_value];
				if -successor_value > best_score then
					-successor_value -> best_score;
					move -> best_path;
					if best_score >= beta then
						return([^best_path ^best_score]);
					endif;
				endif;
			endfor;
			return([^best_path ^best_score]);
		endif;
	endif;
enddefine;


		;;; We must set up all screen objects by describing them as
		;;; instances of a class. ie the yellow counter is described
		;;; as an instance of the class of circles;

vars yelcounter = instance go_circle;
	go_filled	=	true;
	go_bgcolour =	'yellow';
	go_radius	=	25;
	go_live_object = true;
endinstance;

vars redcounter = instance go_circle;
	go_filled	=	true;
	go_bgcolour =	'red';
	go_radius	=	25;
	go_live_object = true;
endinstance;

vars blankbox = instance go_rectangle;
	go_width = 53;
	go_height =53;
	go_bgcolour = 'tan';
	go_fgcolour= 'tan';
	go_filled = true;
endinstance;

vars blankbox2 = instance go_rectangle;
	 go_width = 394;
	 go_height =53;
	 go_bgcolour = 'tan';
	 go_fgcolour= 'tan';
	 go_filled = true;
 endinstance;

;;; set-up variables  here so that they don`t have to be repeated in
;;; every definition.

vars
	infoboxcolour = 'LightSalmon',
	gamemodestring = 'Game mode : Two players',
	skilllevelstring = 'Skill level : Beginner',
	message = 'Waiting for yellow to move',
	game_name = 'connect4.save',
	moves_so_far_string = 'Game moves so far : 0'
;

define grid();
	lvars x,y,count;
	LineSolid -> rc_linestyle;
	- 342.9 -> y;
	for count from 0 to 7 do						;;; use rc-graphic to
		400/7*count -> x ;							;;; draw a simple grid.
		if count /== 7 and count /== 0
		then
			rc_drawline(0,x,400,x);
			rc_drawline(x,0,x,343)
		endif;
	endfor;
enddefine;



;;; bundle together some update modules.
define update_calc_and_redraw(gamestate,game_history,column,colour);
	lvars gamestate,game_history,column,colour;

	update_history(game_history,column) -> game_history;
	calc_gamestate(gamestate,column,colour) -> gamestate;
	draw_new_state(column,gamestate,colour);

	return(gamestate,game_history);
enddefine;

;;; Calculate new game history when a new column is chosen.
define update_history(game_history,column);
	lvars game_history,column;
		 [^^game_history ^(column)] -> game_history;
	return(game_history);
enddefine;

;;; Calculate new gamestate when a new column is chosen.
define calc_gamestate(gamestate,column,colour);
	lvars gamestate,column,sublist,colour;

	gamestate(column) -> sublist;

	if colour == "R" then
		[^^sublist R] -> sublist;
	else
		[^^sublist Y] -> sublist;
	endif;

	sublist -> gamestate(column);
	return(gamestate);
enddefine;

;;; plot single counter when given a column.
define draw_new_state(column,gamestate,colour);
	lvars gamestate,column,yposition,xcoord,ycoord,sublist;

	gamestate(column) -> sublist;
	listlength(sublist) -> yposition;
	400/7*column-28.6 -> xcoord;
	400/7*yposition -28.6 -> ycoord;
	if colour == "R"	 then
		go_centre_to(xcoord,ycoord,redcounter);
		go_copy_object(redcounter) ->redcounter;
		go_draw(pane1,redcounter);
	else
		go_centre_to(xcoord,ycoord,yelcounter);
		go_copy_object(yelcounter) ->yelcounter;
		go_draw(pane1,yelcounter);
	endif;
enddefine;

;;; check to see if the column is allready full.
define check_legal(column,gamestate);
	listlength(gamestate(column)) < 6
enddefine;

;;; finds mouse position and from this expects either
;;; a column select or a box click. - uses lib rc_mouse
define option_check();
	lvars x,y,option_output;
	'false'	 ->	  outputstring;
	false -> option_output;
	repeat
		rc_transxyin(XptValue(rc_window,XtN mouseX),
			XptValue(rc_window,XtN mouseY)) -> y -> x;
		if y < 400	then return(option_output);
		endif;
		if outputstring /= 'false'	then
			outputstring -> option_output;
			return(option_output);
		endif;
	endrepeat;
enddefine;


;;; Makes the counter position follow the mouse pointer and checks
;;; mouse buttons for a selection. Also handles removing a counter
;;; when no column selected.
define input_loop(colour);							;;; requires lib rc_mouse.
	lvars
		x, y, column, colour, old_column, option_output, colposition,
		oldcolposition=100;

	go_centre_to(202,372,blankbox2);
	go_copy_object(blankbox2) -> blankbox2;
	go_draw(pane1,blankbox2);

	false -> button_pos;									  ;;; initialise.
	repeat												   ;;; start loop.
		;;; get mouse
		;;; coordinates.

		rc_transxyin(XptValue(rc_window,XtN mouseX),
			XptValue(rc_window,XtN mouseY)) -> y -> x;

		if y > 400 then
			option_check() -> option_output;
			if not(option_output) then else
				return(option_output)
			endif;
		endif;

		;;; and calc
		x-26.3 ->x;									 ;;; column no.
		intof( x/400*7+0.5) -> column ;
		column+1 -> column ;

		if column > 7 then 7 -> column	endif;		 ;;; Check this
		if column < 1  then 1 -> column	 endif;		 ;;; in the right
		400/7*column-28.6 ->colposition ;					;;; range.

		unless old_column == column then
			if colour == "R"	 and oldcolposition /== colposition then

				go_centre_to(oldcolposition+1,372,blankbox);
				go_copy_object(blankbox) -> blankbox;
				go_draw(pane1,blankbox);
				go_centre_to(colposition,371.4,redcounter);
				go_copy_object(redcounter) -> redcounter;
				go_draw(pane1,redcounter);
			else
				go_centre_to(oldcolposition+1,372,blankbox);
				go_copy_object(blankbox) -> blankbox;
				go_draw(pane1,blankbox);
				go_centre_to(colposition,371.4,yelcounter);
				go_copy_object(yelcounter) -> yelcounter;
				go_draw(pane1,yelcounter);
			endif;
		endunless;
		if button_pos  then
			go_centre_to(202,372,blankbox2);
			go_copy_object(blankbox2) -> blankbox2;
			go_draw(pane1,blankbox2);
			return(column);	 endif; ;;; check mouse.
		column -> old_column;
		colposition -> oldcolposition;
	endrepeat;											  ;;; and repeat.
enddefine;



;;;	 *********************	 MAIN	LOOP ********************
;;;	 *********************				 ********************
;;; Works out whose move next - and calls the relavent module for a column
;;; selection.
;;; Updates moves and information boxes.
;;; and controls the bookmove facility.


define play(mode,game_history,gamestate,colour);
	lvars mode, game_history, gamestate, colour, legal, column;
	update_skill(skill_level);
	update_mode(mode);
	false -> bookmove_flag;
	repeat
		if intof(moves) = moves then	;;; moves is a decimal
			update_moves(moves)
		endif;

		if mode = 'two_players' then
			player_move(legal, colour, game_history, gamestate, column, mode)
				-> (legal, colour, game_history, gamestate, column, mode);
		endif;

		if mode = 'playervcomp' then
			player_move(legal, colour, game_history, gamestate, column, mode)
				-> (legal, colour, game_history, gamestate, column, mode);
			comp_move(colour, game_history, gamestate, column) ->
			(colour, game_history, gamestate, column);
		endif;

		if mode = 'compvplayer' then
			comp_move(colour, game_history, gamestate, column) ->
			(colour, game_history, gamestate, column);
			player_move(legal, colour, game_history, gamestate, column, mode)
				-> (legal, colour, game_history, gamestate, column, mode);
		endif;

		if mode = 'compvcomp' then
			comp_move(colour, game_history, gamestate, column) ->
			(colour, game_history, gamestate, column);
		endif;

		if moves = 21 then
			do_draw(colour, game_history, gamestate, mode);
		endif;

	endrepeat;
enddefine;


;;; Calls for user input - can call option box modules - but still waits
;;; for a column selection.
define player_move(legal,colour,game_history,gamestate,column,mode);
	lvars legal,colour,game_history,gamestate,column,mode;
	false -> legal;
	show_whose_move(colour);

	until legal do
		get_input(colour,game_history,gamestate,mode) ->
		(column,game_history,gamestate,colour,mode);
		check_legal(column,gamestate) -> legal;
	enduntil;

	update_calc_and_redraw(gamestate,game_history,column,colour);
		-> game_history,gamestate;

	if evaluate(gamestate,colour) >= 750 then
		do_win(colour,game_history,gamestate);
	else
		toggle(colour) -> colour;
		moves + 0.5 -> moves;
		return(legal,colour,game_history,gamestate,column,mode);
	endif;
enddefine;


;;; Checks book moves(but not if it has previously failled to find one).
;;; and calls the minimax modules for a computer move.

define comp_move(colour,game_history,gamestate,column);
	lvars colour,game_history,gamestate,column;
	show_whose_move(colour);
	if bookmove_flag then
		make_move(gamestate,colour) -> column;
	else
		bookmove(game_history) -> result;
		if result then
			result -> column;
		else
			true -> bookmove_flag;
			make_move(gamestate,colour) -> column;
		endif;
	endif;

	update_calc_and_redraw(gamestate,game_history,column,colour);
		-> game_history,gamestate;

	if evaluate(gamestate,colour) >= 750  then
		do_win(colour,game_history,gamestate); return();
	else
		toggle(colour) -> colour;
		moves + 0.5 -> moves;
		return(colour,game_history,gamestate,column);
	endif;
enddefine;

 ;;; simple book move database - matches to the game history.
define bookmove(g_history);
	vars count, sublist,g_history,list;

	[
		[[] 4 ]
		[ [1] 4 ]
		[ [2] 4 ]
		[ [5 4] 3]
		[ [3] 4 ]
		[ [4 4 3 2] 3]
		[ [4 4] 3]
		[ [4 4 3] 2]
		[ [4] 5 ]
		[ [5] 4 ]
		[ [6] 4 ]
		[ [7] 4 ]
		[ [4 5 2] 4]
		[ [4 5 2 4] 4]
		[ [4 5 2 4 4] 3]
		[ [4 5 2 4 4 3] 3]
		[ [4 3] 6 ]
		[ [4 5 3] 4]
		[ [4 3 5] 4]
		[ [4 3 6 4 4] 5]
		[ [4 3 6] 4]
		[ [4 3 6 4] 4]
		[ [4 5] 2 ] ] -> list;


	for count from 1 to listlength(list) do
		list(count) -> sublist;
		if g_history matches sublist(1) then
			return(sublist(2));
		endif;
	endfor;
	return(false);
enddefine;


;;; update the information boxes.
define update_moves(moves);
	lvars moves,newstring;
	intof(moves) -> moves;
	'Game moves so far : ' >< moves -> newstring;
	newstring -> go_label(info_box_moves_so_far);
enddefine;

define update_skill(skill_level);
	lvars skill_level,newstring;
	'Skill level : ' >< skill_level -> newstring;	 ;;; concat. a
	newstring -> go_label(info_box_skill);			 ;;; variable with
enddefine;											 ;;; a descriptive
													 ;;; string.

;;; Updates the message box using the 'show_message' procedure.
define show_whose_move(input);
	lvars input;
	switchon input =
	 case  "R" then show_message(['Waiting for red to move']);
	 case  "Y" then show_message(['Waiting for yellow to move']);
	endswitchon;
enddefine;

;;; toggles between red and yellow.
define toggle(colour);
	lvars colour;
	if colour == "R" then "Y"  -> colour;
	else				  "R" -> colour;
	endif;
	return(colour);
enddefine;

;;; flips through messages in a list - displaying each for 1 sec each.
define show_message(list_of_strings);
	lvars no_of_strings,count,list_of_strings;

	listlength(list_of_strings) -> no_of_strings;
	for count from 1 to	 no_of_strings do
		list_of_strings(count) -> go_label(message_box);
		if no_of_strings /= 1 then	syssleep(100) endif;
	endfor;
enddefine;

;;; report a win - then end.
define do_win(colour,game_history,gamestate);
	lvars colour,game_history,gamestate;
	if colour == "R" then
		show_message([ '' ' game over ' ' Red wins this time.']);
	else
		show_message(['' 'game over ' 'Yellow wins this time.']);
	endif;
	'blank' -> outputstring;
	do_end(colour,game_history,gamestate,mode);
enddefine;

;;; check some boxes - and restart.
define do_end(colour, game_history, gamestate, mode);
	lvars colour,game_history,gamestate,mode;
	repeat
		switchon outputstring =
		case 'reset' then
			do_reset(mode);
		case 'load' then
			do_load(game_history)->game_history,skill_level,mode;
			start(mode);
		case 'save' then
			do_save(game_history,skill_level,mode);
		case 'quit' then
			do_quit(colour);
		endswitchon;
		'blank' -> outputstring;
	endrepeat;
enddefine;

;;; report a draw and end.
define do_draw(colour,game_history,gamestate,mode);
	lvars colour,game_history,gamestate;

	show_message(['	 Game drawn' ]);
	do_end(colour,game_history,gamestate,mode);
enddefine;


;;;reinitialise game history/ state and screen.
define clear();
	lvars game_history,gamestate;
	go_refresh();					  ;;; refresh screen.
	grid();
	[] -> game_history;				  ;;; empty history.
	[[][][][][][][]] -> gamestate;	  ;;; empty board.

	return(gamestate,game_history);
enddefine;

;;; do the reset option - just set to two player mode and restart.
;;; remove the comments for a confirmation.
define do_reset(mode);
	repeat
		;;;'blank' -> outputstring;
		;;;	   show_message(['Click Reset to confirm' 'or forwards to continue']);
		;;;	   if outputstring = 'reset' then
		0 -> moves;
		start('two_players');
		;;;	  endif;
		;;; if	outputstring = 'forwards' then show_whose_move(colour);
		;;;return(); endif;
	endrepeat;
enddefine;


;;; set-up initial	conditions and reset all values.
;;; then play.
define start(mode);
	lvars gamestate,game_history,mode;
	"Y" -> colour;
	clear() -> (gamestate,game_history);
	play(mode,game_history,gamestate,colour);
enddefine;

;;; set-up initial conditions for player/comp mode.This requires user
;;; input to decide who plays which colour.
;;; also works out who plays first.
define start_playervcomp(mode,game_history,gamestate,colour);
	lvars mode,game_history,gamestate,colour,who;

	'playervcomp' -> mode;
	'blank' -> outputstring;
	'	Red' -> go_label(forwards);
	'	Yellow' -> go_label(backwards);

;;;	repeat 1 times
		'tomato' -> go_bgcolour(forwards);		  ;;; highlight boxes
		'tomato' -> go_bgcolour(backwards);		  ;;; so they`re noticed.
;;;	endrepeat;

	if colour == "R" then ' Red to move next.' -> who;
	else 'Yellow to move next.' -> who;
	endif;

	until outputstring /= 'blank' do
		show_message([ 'Choose red or yellow' ^(who)]);
	enduntil;

	if colour == "Y" then
		if	outputstring = 'forwards' then 'compvplayer' -> mode ;
		elseif outputstring = 'backwards' then 'playervcomp' -> mode ;
		endif;
	endif;
	if colour == "R" then
		if	outputstring = 'forwards' then 'playervcomp' -> mode ;
		elseif outputstring = 'backwards' then 'compvplayer' -> mode ;
		endif;
	endif;
	'pink3' -> go_bgcolour(forwards);
	'pink3' -> go_bgcolour(backwards);
	'forwards' -> go_label(forwards);
	'backwards' -> go_label(backwards);
	play(mode,game_history,gamestate,colour);
enddefine;

;;; Executes the module coresponding to the option chosen.
define test_boxes(result, game_history, gamestate, colour, mode);
	lvars result;
	switchon result =
		case 'reset' then
			do_reset(mode);
		case 'save'  then
			do_save(game_history, skill_level, mode);
		case 'load' then
			do_load(game_history, colour, mode) ->
			(game_history, skill_level, mode, colour);
		case 'hint' then
			do_hint(colour, gamestate);
		case 'two_players' then
			start('two_players');
		case 'playervcomp' then
			start_playervcomp(mode, game_history, gamestate, colour);
		case 'compvcomp' then
			play('compvcomp',game_history,gamestate,colour);
		case 'rules' then
			do_rules(colour);
		case 'quit' then
			do_quit(colour);
		case 'skill_level' then
			change_skill(skill_level)-> skill_level;
		case 'backwards' then
			do_backwards(gamestate, game_history, colour, mode)
				-> (game_history, gamestate, colour);
	endswitchon;
	'blank' -> outputstring;
	return(game_history,gamestate,colour,mode);
enddefine;

;;; toggle the skill level between three values - and update.
define change_skill(skill_level);
	lvars skill_level,newlevel;

	switchon skill_level =
		case 'Beginner' then 'Intermediate' -> newlevel;
		case 'Intermediate' then 'Expert' -> newlevel;
		case 'Expert' then 'Beginner' -> newlevel;
	endswitchon;

	newlevel -> skill_level;
	update_skill(skill_level);
	return(skill_level);
enddefine;

;;; returns the selected column or executes a game option.
define get_input(colour,game_history,gamestate,mode);
	lvars colour,game_history,gamestate,mode;
	repeat
		input_loop(colour) -> result;
		if isinteger(result) then return(result,game_history,gamestate,colour
				,mode);
		endif;
		test_boxes(result,game_history,gamestate,colour,mode)
			->(game_history,gamestate,colour,mode);
	endrepeat;
enddefine;

;;; updates the screen display to show the current game mode.
define update_mode(mode);
	lvars mode,newstring;
	switchon mode =
	case 'two_players' then
		'Game mode: Two players ' -> newstring;
	case 'compvcomp'   then
		'Game mode: Two computers' -> newstring;
	case 'playervcomp' or 'compvplayer' then
		'Game mode: player-comp' -> newstring;
	endswitchon;

	newstring  -> go_label(info_box_mode);
enddefine;

;;; Steps through a list of game rules - returns at the end
;;; or when the user selects any box.
define do_rules(colour);
	lvars biglist,count,size;

	[' Here are the game rules' 'Each player starts with'
		'21 red/yellow counters' 'choose a column 1-7'
		'drop counters alternately' 'to fill the board.'
		'the aim is to . . . ' 'make a line of four' 'in your colour '
		'either horizontally' 'vertically' 'or diagonally' '' ] -> biglist;

	listlength(biglist) -> size;
	1 -> count;
	'blank' -> outputstring;
	until count == size or outputstring /= 'blank' do
		show_message([^(biglist(count))]);
		syssleep(175);								  ;;; pause to read
		1 + count -> count;
	enduntil;
	show_whose_move(colour);
enddefine;

;;; Checks for confirmation and then either exits or continues.
define do_quit(colour);
	lvars colour, recorded;

	go_label(message_box) -> recorded;
	repeat
		'blank' -> outputstring;
		show_message(['Click quit  to confirm' 'or forwards to continue']);

		if outputstring = 'quit' then
			go_destroy();
			while stacklength() > 0 do
				;;; print out any junk left on stack
				=>
			endwhile;
			exitfrom(start_up);
		elseif outputstring = 'forwards' then
			show_message([^(recorded)]);
			return();
		endif;

	endrepeat;
enddefine;


;;; requests filename/confirmation and saves
define do_save(game_history,skill_level,mode);
	lvars filename,count,list,x,y,colflag='low',
		mode,countdown,game_history,skill_level,mode;

	go_destroy_object(info_box_moves_so_far);

	go_add_to(text_input_box,pane1);

	['Alter filename	 --->' 'while cursor is in the box '
			'and click SAVE to confirm'] -> list;

	'blank' -> outputstring;

	while outputstring = 'blank' do
	 	for count from 1 to listlength(list) do
	 		show_message([%list(count)%]);
	 		0 -> countdown;

			;;; pause loop - but still checks mouse coords

	 		while outputstring = 'blank' and countdown	 <	3000  do
	 			rc_transxyin(XptValue(rc_window,XtN mouseX),
					XptValue(rc_window,XtN mouseY)) -> y -> x;

				;;;	 Only highlight the box if the mouse pointer
				;;;	 is in it AND it is not allready highlighted.

				if x < 395 and x > 200 and y > 405 and y < 435 then
					if colflag = 'low' then
						'RosyBrown1' -> go_bgcolour(text_input_box);
						'high' -> colflag;
					endif;
				else
					if colflag = 'high' then
					 	'mint cream' -> go_bgcolour(text_input_box);
					 	'low' -> colflag;
					endif;
				endif;
				1+countdown -> countdown;
			endwhile;

		endfor;
	endwhile;

	if outputstring = 'save' then
		go_label(text_input_box) -> filename;
		consave(filename,game_history,skill_level,mode);
	endif;
	;;; after save procedure set up display as before.

	go_destroy_object(text_input_box);
	go_add_to(info_box_moves_so_far,pane1);
	show_message(['Returning to the game']);
enddefine;

;;; Prompts for filename input by passing a message and highlighting
;;; the text box when the mouse is in it.The relevant game data is then
;;; retrieved from the load procedure.

define do_load(game_history,colour,mode);
	lvars filename,count,list,x,y,colflag='low',countdown,
		colour,mode;

	go_destroy_object(info_box_moves_so_far);

	go_add_to(text_input_box,pane1);

	['Alter filename	 --->' 'while cursor is in the box '
			'and click LOAD to confirm'] -> list;

	'blank' -> outputstring;
	while outputstring = 'blank' do
	 	for count from 1 to listlength(list) do
		 	show_message([%list(count)%]);
	 		0 -> countdown;
	 		while outputstring = 'blank' and countdown	 <	3000  do
	 			rc_transxyin(XptValue(rc_window,XtN mouseX),
					XptValue(rc_window,XtN mouseY)) -> y -> x;

				if x < 395 and x > 200 and y > 405 and y < 435 then
					if colflag = 'low' then
						'RosyBrown1' -> go_bgcolour(text_input_box);
						'high' -> colflag;
					endif;
				else
					if colflag = 'high' then
					 	'mint cream' -> go_bgcolour(text_input_box);
					 	'low' -> colflag;
					endif;
				endif;
				1+countdown -> countdown;
			endwhile;
		endfor;
	endwhile;
	if outputstring = 'load' then
		go_label(text_input_box) -> filename;
		if	check_filed(filename) then
			clear() -> (game_history,gamestate);

			load_game(filename) -> (game_history,skill_level,mode);

			;;;	 We must now set up the game from this data.
			gamestate_from_history(game_history,"Y") -> gamestate;
			plot_gamestate(gamestate);
			update_skill(skill_level);
			calc_moves(game_history) -> moves;
			go_destroy_object(text_input_box);
			go_add_to(info_box_moves_so_far,pane1);
			show_message(['Returning to the game']);
			update_moves(intof(moves));
			update_mode(mode);

			if intof(moves) = moves then "Y" -> colour;
			else "R" -> colour;
			endif;

			play(mode,game_history,gamestate,colour);
		else
		show_message(['Filename not found' 'Filename not found']);
		endif;
	endif;
	go_destroy_object(text_input_box);
	go_add_to(info_box_moves_so_far,pane1);
	show_message(['Returning to the game']);
	return(game_history, skill_level, mode, colour);
enddefine;

;;; Works out the integer game moves from the game history.
define calc_moves(game_history);
	lvars game_history,dum;
	listlength(game_history) * 0.5 -> dum;
	return(dum);
enddefine;

;;; blanks out a counter on the screen by covering it with a background
;;; coloured square.The position on the screen is calculated by the
;;; column chosen and the current game state.
define remove_counter(column,gamestate);
 	lvars gamestate, column, yposition, xcoord, ycoord, sublist;

	gamestate(column) -> sublist;
	listlength(sublist)  -> yposition;
	400/7*column-28.6 -> xcoord;
	400/7*yposition -28.6 -> ycoord;
	go_centre_to(xcoord,ycoord,blankbox);
	go_copy_object(blankbox) ->blankbox;
	go_draw(pane1,blankbox);

enddefine;


;;; (1) - sets up display so that a "PLAY" box is created and highlighted.
;;; (2) - highlights the "forwards" and "backwards" boxes.
;;; (3) - goes back one move. All relavent game parameters are updated
;;;		  to reflect the new state(moves made, game state ,history ).
;;; (4) - Enters a loop within which the forwards and backwards operations
;;;		  are performed . Exits when "PLAY" is selected.
;;; (5) - Resets graphical display.
;;; (6) - Returns new game parameters.

;;; The mechanism used here is to use a pointer in the game history vector.
;;; in this way a move backwards does not destroy data on later moves.

define do_backwards(gamestate,game_history,colour,mode);
	lvars index,gamelength,history,firstcolour,
		game_history,gamestate,the_column,colour,compcolour;

	;;; highlight boxes.
	'tomato' ->> go_bgcolour(forwards) ->> go_bgcolour(backwards) ->>
	go_bgcolour(reset);

	if colour == "R" then "Y" -> compcolour;	 ;;;  works out which
	else				 "R" -> compcolour;	 ;;;  colour the computer
	endif;									 ;;; is playing.
	;;; clear top row of any counters.
	go_centre_to(202, 372, blankbox2);
	go_copy_object(blankbox2) -> blankbox2;
	go_draw(pane1, blankbox2);

	show_message(['click PLAY to continue']);
	'PLAY ' -> go_label(reset);
	listlength(game_history) ->> gamelength -> index;
	if gamelength/2 /= intof(gamelength/2) then
		if colour == "R" then "Y" -> firstcolour;
		else "R" -> firstcolour;
		endif;
	else
		if colour == "R" then "R" -> firstcolour;
		else "Y" -> firstcolour;
		endif;
	endif;

	while outputstring = 'blank'
	or	outputstring = 'forwards'
	or	outputstring = 'backwards' do
		if outputstring ='forwards' and index /= gamelength then
			index + 1 -> index;
			game_history(index) -> the_column;
			allbutlast((gamelength-index),game_history) -> history;
			gamestate_from_history(history,firstcolour) -> gamestate;
			draw_new_state(the_column,gamestate,colour);
			toggle(colour) -> colour;
			moves + 0.5 -> moves;
		endif;

		if outputstring = 'backwards' and index /= 0 then
			game_history(index) -> the_column;
			remove_counter(the_column,gamestate);
			toggle(colour) -> colour;
			moves - 0.5 -> moves;
			index-1 -> index;

			allbutlast((gamelength-index),game_history) -> history;
			gamestate_from_history(history,firstcolour) -> gamestate;
			update_moves(intof(moves));

		endif;

		if	outputstring /= 'blank' then
			update_moves(intof(moves));
		endif;

		'blank' -> outputstring;
		'tomato' ->> go_bgcolour(forwards)
			->> go_bgcolour(backwards)
			->> go_bgcolour(reset);
	endwhile;

	;;; restore original colours.
	'reset' -> go_label(reset);
	'pink3' ->> go_bgcolour(forwards) ->> go_bgcolour(backwards) ->>
	go_bgcolour(reset);

	show_whose_move(colour);
	gamestate_from_history(history,firstcolour) -> gamestate;

	;;; work out who plays next and set mode accordingly.
	if (colour = compcolour and mode = 'compvplayer') or
		(colour = compcolour and mode = 'playervcomp')
	then
		comp_move(colour, history, gamestate, column) ->
			(colour, history, gamestate, column);
	endif;
	return(history, gamestate, colour)
enddefine;


;;; constructs the full gamestate from the game history.
define gamestate_from_history(a_history, colour);
	lvars dummy_state, count, column, colour, a_history;
	[[][][][][][][]] -> dummy_state;
	for count from 1 to listlength(a_history) do
		a_history(count)-> column;
		[^^(dummy_state(column)) ^colour] -> dummy_state(column);
		toggle(colour) -> colour;
	endfor;
	return(dummy_state);
enddefine;

;;; calls the minimax at the current skill level and flashes a counter
;;; at this position.
define do_hint(colour,gamestate);
	lvars colour,gamestate,column;
	show_message([' Waiting for a hint. ']);
	make_move(gamestate,colour) -> column;
	show_message([ ' Try this position']);
	calc_gamestate(gamestate,column,colour);
	repeat 18 times
		draw_new_state(column, gamestate, colour);
		syssleep(20);
		remove_counter(column, gamestate);
		syssleep(10);
	endrepeat;
	allbutlast(1, gamestate(column)) -> gamestate(column);
enddefine;

;;; Uses the datafile command to save the game state.
define consave(filename, g_history, skill, mode);
	lvars g_history, skill, mode, filename;
	 [^g_history ^skill ^mode] -> datafile(filename);
enddefine;

;;; loads a filename into a list and deconstructs it into the game parameters.
;;; Name changed by A.S. to prevent clash with object loadgame
define load_game(filename);
 	lvars list, g_history, skill, mode;

	datafile(filename) -> list;
	list(1) -> g_history;
	list(2) -> skill;
	list(3) -> mode;
 	return(g_history, skill, mode);
enddefine;

;;; Makes use of 'sysfileinode' which returns 'false' if a filename does
;;; not exist.
;;; Could just do
;;; global constant procedure check_filed = sysfileinode;
define check_filed(filename);
	lvars filename;
	if sysfileinode(filename) == false then return(false);
	else return(true);
	endif;
enddefine;

;;; Define some of the screen objects.
vars
	boxcolour = 'pink3'
	,box_highlightcol = 'violet red';

define window_up();
	650 -> rc_window_ysize;
	650 -> rc_ymax;
	vars pane1 = instance go_rc_window;
	  go_window_width	=500;
	  go_window_height	=650;
	  go_bounding_height = 650;				 ;;; set up window.
	  go_bounding_width = 500;
	  go_xorigin		=50;
	  go_yorigin		=600;
	  go_bgcolour = 'navajo white';
	 endinstance;
enddefine;

define objects();
	vars yelcounter =
	instance go_circle;
	   go_filled   =   true;
	   go_bgcolour =  'yellow';			   ;;; define the yellow counter
	   go_radius   =   25;				   ;;; as a filled yellow circle.
	   go_live_object = true;
	endinstance;

	vars redcounter =
	instance go_circle;
	  go_filled	  =	  true;
	  go_bgcolour =	  'red';			   ;;; and a red one.
	  go_radius	  =	  25;
	  go_live_object = true;			   ;;; 'live' means not editable
	endinstance;						   ;;; with the mouse.

enddefine;


;;; Makes copies of the counters at places on the board described
;;; by the game state.

define plot_gamestate(gamestate);
	lvars xcount, ycount, collist, heightofcol;

	for xcount from 1 to 7 do
		listlength(gamestate(xcount)) -> heightofcol;
		gamestate(xcount) -> collist;
		for ycount from 1 to heightofcol do
			if collist(ycount) == "R"	 then
				go_centre_to(400/7*(xcount-1)+28.6
					,400/7*(ycount-1)+28.6,redcounter);
				go_copy_object(redcounter) -> redcounter;
				go_draw(pane1,redcounter);
			endif;
			if collist(ycount) == "Y"	 then
				go_centre_to(400/7*(xcount-1)+28.6
					,400/7*(ycount-1)+28.6,yelcounter);
				go_copy_object(yelcounter) -> yelcounter;
				go_draw(pane1,yelcounter );
			endif;
		endfor;
	endfor;
enddefine;


define :class sensitives;
	isa go_mouse_sensitive go_oblong;	;;;	 define a class of mouse
enddefine;								;;;	 sensitive oblongs.

define :class labelbox;
	isa go_labelled sensitives;			;;; one of labelled , sensitive
enddefine;								;;; oblongs.

define :class input_box;
	isa	 go_text_input	go_oblong;		;;; and one text input oblong.
enddefine;


;;; Define methods to highlight boxes and change the 'outputstring'
;;; variable.
define :method highlightreset( event_data , reset :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(reset);
enddefine;

define :method dehighlightreset(event_data,reset :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(reset);
	'reset' -> outputstring;
enddefine;

define :method highlightsave( event_data , save	 :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(save);
enddefine;

define :method dehighlightsave(event_data, save:labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(save);
	'save' -> outputstring;
enddefine;

define :method highlightloadgame(event_data, loadgame:labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(loadgame);
enddefine;

define :method dehighlightloadgame(event_data,loadgame :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(loadgame);
	'load' -> outputstring;
enddefine;

define :method highlighthint( event_data , hint	 :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(hint);

enddefine;

define :method dehighlighthint(event_data,hint :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(hint);
	'hint' -> outputstring;
enddefine;

define :method highlightforwards( event_data , forwards	 :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(forwards);
enddefine;

define :method dehighlightforwards(event_data,forwards :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(forwards);
	'forwards' -> outputstring;
enddefine;

define :method highlightbackwards( event_data ,backwards  :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(backwards);
enddefine;

define :method dehighlightbackwards(event_data,backwards :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(backwards);
	'backwards' -> outputstring;
enddefine;

define :method highlighttwo_players( event_data ,twoplayers	 :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(twoplayers);
enddefine;

define :method dehighlighttwo_players(event_data,two_players :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(two_players);
	'two_players'-> outputstring;
enddefine;

define :method highlightcompvcomp( event_data ,compvcomp  :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(compvcomp);
enddefine;

define :method dehighlightcompvcomp(event_data,compvcomp :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(compvcomp);
	'compvcomp' -> outputstring;
enddefine;

define :method highlightplayervcomp( event_data ,playervcomp  :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(playervcomp);
enddefine;

define :method dehighlightplayervcomp(event_data,playervcomp :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(playervcomp);
	'playervcomp' -> outputstring;
enddefine;

define :method highlightquit( event_data ,quit	:labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(quit);
enddefine;

define :method dehighlightquit(event_data,quit :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(quit);
	'quit' -> outputstring;
enddefine;

define :method highlightrules( event_data ,rules  :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(rules);
enddefine;

define :method dehighlightrules(event_data,rules :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(rules);
	'rules' -> outputstring;
enddefine;

define :method highlightskill_level( event_data ,skill_level  :labelbox);
	lvars event_data, labelbox;
	box_highlightcol -> go_bgcolour(skill_level);
enddefine;

define :method dehighlightskill_level(event_data,skill_level :labelbox);
	lvars event_data,labelbox;
	boxcolour  -> go_bgcolour(skill_level);
	'skill_level' -> outputstring;
enddefine;

define :method click_check(event_data,rect :sensitives);
	lvars event_data,sensitives;
	'true' -> button_pos;
enddefine;



define set_up_boxes(pane1, boxcolour, boxhighlightcol);

	lvars x,y,count;
	vars rect =
	instance sensitives;
		go_bgcolour	 = 'tan';
		go_filled	 = true;
		go_width	 = 400;					 ;;; set up board
		go_height	 = 400;					 ;;; background.
		go_live_object =true;
	endinstance;

	go_centre_to(200,200,rect);				;;; draw board background.

	;;; Describe the graphical screen objects as instances of the above classes.
	vars reset =
	instance labelbox;
		go_width = 95;
		go_height= 30;
		go_label ='Reset';
		go_bgcolour=boxcolour;
		go_live_object =true;		  ;;; 'reset' box.
		go_filled =true;
		go_yloc = 550;
		go_xloc = 0;
		go_label_x_offset=30;
		go_label_y_offset=10;
	endinstance;

	vars save =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=32;		  ;;; 'save' box.
		go_label_y_offset=8;
		go_label='save';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=515;
		go_xloc=0;
	endinstance;

	vars loadgame  =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=32;
		go_label_y_offset=8;			;;; 'load' box.
		go_label='load';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=480;
		go_xloc=0;
	endinstance;

	vars quit  =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=32;
		go_label_y_offset=8;			;;; 'quit' box.
		go_label='quit';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=550;
		go_xloc=300;
	endinstance;

	vars hint  =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=32;
		go_label_y_offset=8;			;;; 'hint' box.
		go_label='hint';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=550;
		go_xloc=100;
	endinstance;

	vars forwards  =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;			;;; 'forwards' box.
		go_label='forwards';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=515;
		go_xloc=100;
	endinstance;

	vars backwards	=
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=14;
		go_label_y_offset=8;
		go_label='backwards';			;;; 'backwards' box.
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=480;
		go_xloc=100;
	endinstance;

	vars two_players  =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=8;
		go_label_y_offset=8;			;;; 'two players' box.
		go_label='Two players';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=550;
		go_xloc=200;
	endinstance;

	vars compvcomp =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=8;
		go_label_y_offset=8;			;;; 'Two comp.s' box.
		go_label='Two comp.s';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=515;
		go_xloc=200;
	endinstance;

	vars playervcomp =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=4;
		go_label_y_offset=8;			;;; 'Player v comp' box.
		go_label='Player v comp.';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=480;
		go_xloc=200;
	endinstance;

	vars rules=
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=30;
		go_label_y_offset=8;			 ;;; 'rules' box.
		go_label='rules';
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=515;
		go_xloc=300;
	endinstance;

	vars skill_level =
	instance labelbox;
		go_width=95;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;
		go_label='skill level';			 ;;; 'skill level' box.
		go_bgcolour=boxcolour;
		go_live_object=true;
		go_yloc=480;
		go_xloc=300;
	endinstance;

	vars info_box_mode =
	instance labelbox;
		go_width=195;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;			 ;;; mode information box.
		go_label_y_offset=8;
		go_label=gamemodestring;
		go_bgcolour=infoboxcolour;
		go_live_object=true;
		go_yloc=440;
		go_xloc=0;
	endinstance;


	vars info_box_skill =
	instance labelbox;
		go_width=195;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;
		go_label=skilllevelstring;		;;; skill level information box.
		go_bgcolour=infoboxcolour;
		go_live_object=true;
		go_yloc=440;
		go_xloc=200;
	endinstance;

	vars message_box =
	instance labelbox;
		go_width=195;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;			 ;;; message box.
		go_label=message;
		go_bgcolour='mint cream';
		go_live_object=true;
		go_yloc=405;
		go_xloc= 00;
	endinstance;

	vars info_box_moves_so_far =
	instance labelbox;
		go_width=195;
		go_height=30;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;
		go_label=moves_so_far_string;	 ;;; game moves information box.
		go_bgcolour=infoboxcolour;
		go_live_object=true;
		go_yloc=405;
		go_xloc=200;
	endinstance;

	vars text_input_box	 =
	instance input_box;
		go_width=195;
		go_height=30;
		go_rounding = 8;
		go_filled=true;
		go_label_x_offset=20;
		go_label_y_offset=8;			   ;;; text input box.
		go_label=game_name;
		go_bgcolour='mint cream';
		go_live_object=true;
		go_yloc=405;
		go_xloc=200;
	endinstance;

	go_add_to(rect,pane1);				   ;;; Draw them all in pane1.
	go_add_to(reset,pane1);
	go_add_to(save,pane1);
	go_add_to(loadgame,pane1);
	go_add_to(quit,pane1);
	go_add_to(hint,pane1);
	go_add_to(forwards,pane1);
	go_add_to(backwards,pane1);
	go_add_to(two_players,pane1);
	go_add_to(compvcomp ,pane1);
	go_add_to(playervcomp,pane1);
	go_add_to(rules,pane1);
	go_add_to(skill_level,pane1);
	go_add_to(info_box_moves_so_far,pane1);
	go_add_to( info_box_mode,pane1);
	go_add_to(message_box,pane1);
	go_add_to(info_box_skill,pane1);


	;;; And attach the methods to a mouse action.
	click_check -> go_mouse_down_action(1,rect);	;;; the '1' is the left
	;;; mouse button.
	highlightreset -> go_mouse_down_action(1,reset);
	dehighlightreset ->go_mouse_up_action(1,reset);

	highlightsave -> go_mouse_down_action(1,save);
	dehighlightsave ->go_mouse_up_action(1,save);

	highlightloadgame -> go_mouse_down_action(1,loadgame);
	dehighlightloadgame ->go_mouse_up_action(1,loadgame);

	highlighthint -> go_mouse_down_action(1,hint);
	dehighlighthint ->go_mouse_up_action(1,hint);

	highlightforwards -> go_mouse_down_action(1,forwards);
	dehighlightforwards ->go_mouse_up_action(1,forwards) ;

	highlightbackwards -> go_mouse_down_action(1,backwards);
	dehighlightbackwards ->go_mouse_up_action(1,backwards);

	highlighttwo_players -> go_mouse_down_action(1,two_players);
	dehighlighttwo_players ->go_mouse_up_action(1,two_players);

	highlightcompvcomp -> go_mouse_down_action(1,compvcomp);
	dehighlightcompvcomp ->go_mouse_up_action(1,compvcomp);

	highlightplayervcomp -> go_mouse_down_action(1,playervcomp);
	dehighlightplayervcomp ->go_mouse_up_action(1,playervcomp) ;

	highlightquit -> go_mouse_down_action(1,quit);
	dehighlightquit ->go_mouse_up_action(1,quit);

	highlightrules -> go_mouse_down_action(1,rules);
	dehighlightrules ->go_mouse_up_action(1,rules);

	highlightskill_level -> go_mouse_down_action(1,skill_level);
	dehighlightskill_level ->go_mouse_up_action(1,skill_level);
	return(message_box,info_box_moves_so_far,info_box_skill,
		info_box_mode,text_input_box,forwards,backwards,reset,
		text_input_box);
enddefine;

;;; Sets up the definitions , pops up a window and passes out the object
;;; names. NB. This is not the top level procedure. See start_up
define setup();
	vars
		boxcolour ='pink3',
		box_highlightcol='tomato';

	window_up();
	go_default_pane -> pane1;
	set_up_boxes(pane1,boxcolour,boxhighlightcol) -> (message_box,
	info_box_moves_so_far,info_box_skill,info_box_mode,text_input_box,forwards
	,backwards,reset,text_input_box);
	grid();
	return(message_box,info_box_moves_so_far,info_box_skill,
		 info_box_mode,text_input_box,forwards,backwards,
		 reset,text_input_box);
enddefine;


pr('\nTo run connect4, give the pop-11 command\n\n\tstart_up();\n');
