/* --- Copyright University of Birmingham 1996. All rights reserved. ------
 > File:            $poplocal/local/lib/gramdemo.p
 */

/*  --- Copyright University of Sussex 1986.  All rights reserved. ---------
 >  File:           $poplocal/local/lib/gramdemo.p
	;;; Extended version of
 >  File:           C.all/lib/lib/gramdemo.p
 >  Purpose:        demonstration of parsing sentences (to do with a kitchen)
 >  Author:         A.Sloman 1983 (see revisions)
 >            		Revised Aaron Sloman, Oct 14 1996
 >  Documentation:  $usepop/pop/lib/demo/kitchen.lis
 >  Related Files:  $usepop/pop/lib/demo/mkgram and saved image
 */

#_TERMIN_IF DEF POPC_COMPILING

if popheader then pr(popheader); pr(newline); '' -> popheader endif;
uses showtree;

pr('please wait\n');
uses grammar;
uses type;

define getline(pop_readline_prompt) -> result;
	;;; Like readline, but ignore apostrophes. Convert to lower case.
	dlocal popnewline   proglist;
	true -> popnewline;
	define lconstant inchar() -> char;
		lvars char;
		charin() -> char;
		while char == `'` or char == ``` do
			charin() -> char;
		endwhile;
		if isuppercode(char) then char + 32 -> char endif;
	enddefine;
	pdtolist(incharitem(inchar) )-> proglist;
	readline() -> result;
	if result /== [] and last(result) == "." then
		allbutlast(1, result) -> result
	endif;
enddefine;


define setmeaning(item,word);
	lvars x;
	Wdprops(word) -> x;
	if x == undef or x == [] then item
	elseif isword(x) then [^item ^x]
	else [^item ^^x]
	endif
	-> Wdprops(word)
enddefine;


define testmember(word,list) -> cat;
	;;; if word is an initial segment of an element of the list,
	;;; return the element, otherwise false
	false-> cat;
	applist(list,
		procedure(x);
			if issubstring(word,1,x) == 1 then
				x; exitfrom(testmember)
			endif
		endprocedure)
enddefine;

define checktype(word, Lexicon);
	lvars cat list x;
	if Wdprops(word) == [] then
		printf(word,
		 '\n\nWHAT TYPE OF WORD IS: %p ?\nPlease answer one of:\n\n');
		 [%for cat in Lexicon do
				hd(cat) -> x;
				unless issubstring('that',1,x)
				or (issubstring('verb',1,x) and strmember(`_`,x)) then
					 x
				endunless
			 endfor%] -> list;
		list ==>
		until
			(pr('\n\n(You can abbreviate to 3 letters). Press the RETURN button if you don\'t know.');
			getline('\n\r\n\rOtherwise type your answer and press return: ') -> x;

			if x == [] then
				pr('I assume you don\'t know. Will see what I can do.\n');
				return
			else
				hd(x) -> cat;
				testmember(cat,list) ->> cat
			endif)
		do
			pr('Please cat one of:\n\t ');
			list ==>
		enduntil;
		setmeaning(cat,word)
	endif;
enddefine;

define ends_ing(x);
	if length(x) < 3 then false
	else
		issubstring('ing', length(x)-2, x) == length(x) - 2
	endif
enddefine;


define checksentence(list,Lexicon);
	lvars x y temp;
	list -> temp;
	while temp matches ! [== ?x:be_word ?y:ends_ing ??temp] do
		setmeaning("adjective",y);
		printf(y, '\Treating %p as an adjective\n');
	endwhile;
	applist (list,checktype(%Lexicon%));
	true        ;;; may be generalised
enddefine;


vars gram_a lex_a;;

[
;;; sentence formats
[SENTENCE
	[NOUN_PHRASE VERB_PHRASE]
	[NOUN_PHRASE verb NOUN_PHRASE that_word SENTENCE]
	[NOUN_PHRASE verb that_word SENTENCE]
	[SENTENCE  conjunction SENTENCE]
	]
;;; noun phrases
[NOUN_PHRASE
	[name]
	[pronoun]
	[SIMPLE_NOUN_PHRASE]
	[NOUN_PHRASE PREPOSITION_PHRASE]
	[NOUN_PHRASE conjunction NOUN_PHRASE]
	]
;;; simple noun phrase
[SIMPLE_NOUN_PHRASE
	[determiner noun]
	[determiner QUALIF_NOUN]
	[QUALIF_NOUN]
	]
;;; qualified noun
[QUALIF_NOUN
	[noun]
	[noun noun]
	[adjective QUALIF_NOUN]
	[possessive QUALIF_NOUN]
	]
;;; prepositional phrases
[PREPOSITION_PHRASE
	[preposition NOUN_PHRASE]
	]
;;; verb phrases
[VERB_PHRASE
	[had VERB_PHRASE]
	[be_word adjective]
	[be_word adjective NOUN_PHRASE]
	[be_word preposition NOUN_PHRASE]
	[verb_prep_np preposition NOUN_PHRASE]
	[verb_np_prep_np NOUN_PHRASE preposition NOUN_PHRASE]
	[verb_np_prep NOUN_PHRASE preposition]
	[verb_np NOUN_PHRASE]
	[verb_np_np NOUN_PHRASE NOUN_PHRASE]
	[intransitiveverb]
	[verb NOUN PHRASE]
	[verb OBJ_PHRASE]
	[verb to verb_inf]
	[verb NOUN_PHRASE to verb_inf ]
	[verb NOUN_PHRASE to verb]
	[verb NOUN_PHRASE to verb_inf OBJ_PHRASE]
	[verb NOUN_PHRASE to verb OBJ_PHRASE]
	[verb to verb_inf OBJ_PHRASE]
	]
[OBJ_PHRASE
	[adjective]
	[adjective NOUN_PHRASE]         ;;;adjectives include participles
	[preposition NOUN_PHRASE]
	[NOUN_PHRASE preposition]
	[NOUN_PHRASE preposition NOUN_PHRASE]
	[NOUN_PHRASE]
	[NOUN_PHRASE NOUN_PHRASE]
	[preposition]
	]
] -> gram_a;

[
	[name Joe Mary joe mary ]
	[pronoun she he you it i I they them we her me him]
	[determiner the a an each every some ]
	[noun
		apple apples biscuit biscuits butter cake carrots cheese
		chip chips chocolate coffee cocoa cookies egg eggs flour food
		grub herbs juice lemon milk orange peas pepper
		potatoes powder rice salt soup squash sugar tea water

		bag bowl bottle can cup fork jug knife pan pot saucepan spoon
		strainer saucer

		boy chef cook girl man waiter woman
		chair cooker cupboard floor fridge hob shelf sink table tap taps
		fire gas
		kitchen room light mess
		mine yours hers his its theirs joes marys]

	[adjective smelly old new fresh sweet sour bitter tasty
		white green red sticky wet raw rancid hot cold warm
		mine hers his yours theirs]

	[possessive joes marys his her its their my your our]
	[be_word were was got became wasnt is isnt werent]
	[that_word that how where when whether if]
	[preposition on in out under over at by with from to of onto into
		below above]
	[conjunction and while when because]
	[verb_prep_np looked poked climbed stood fell wept switched turned
		weighed]
	[verb_np_prep turned switched picked]
	[verb_np_prep_np threw stuck gave joined took stirred mixed poured
		shook sprinkled fetched lifted removed stole put smeared baked
		cooked]
	[verb_np_np baked made cooked cut buttered poured]
	[verb_np stirred baked cooked mixed liked hated poured caught lit shut
		opened roasted boiled dropped drank ate drank cut washed cleaned
		weighed made had]
	[intransitiveverb cooked poured stirred ate wept watched slept]
	[verb weighed struck boiled washed poured turned stood had told asked
		   thought wondered supposed remembered
		switched threw]    ;;; for multiple uses of these verbs
	[verb_inf be bake pour stir make cut open cook drink eat shut give
		pick light mix shake fetch turn put drop strike turn cook
		wash dry clean weigh]
	[adverb slowly carefully ]
] -> lex_a;


setup(gram_a,lex_a);

vars first, Last;
define test();
	lvars x finished ;
	dlocal proglist, vedautowrite;
	lvars first, Last;
	pdtolist(incharitem(charin)) -> proglist;
	false ->> vedautowrite ->> finished -> x;
	until finished do
		unless x then
			type('$usepop/pop/lib/demo/kitchen.lis');
			getline('Guess what went on in the kitchen?\r\n? ') -> x;
		endunless;
		;;; remove full stops
		if hd(x) = "setpop" then
			;;; for debugging
			setpop -> interrupt; sysprmishap -> prmishap; true -> popsyscall;
			setpop()
		elseif x = [bye] or x == termin then
			true -> finished
		else
			while x  matches ! [??first . ??Last] do
				[^^first ^^Last] -> x
			endwhile;
			if checksentence(x,lex_a) and
				(SENTENCE(x) ->> x)
			then
				pr('Analysed it. Now for a diagram\n');
				showtree_mess(x);
			else
				'Not analysable according to this grammar' =>
			endif;
			getline('Want to try another? (y/n) (end with RETURN)\r\n')-> x;
			if length(x) == 1 then
				hd(x) -> x;
				if member (x, [bye no nope n]) then true -> finished
				else false -> x;
				endif;
			endif;
		endif;
	enduntil;
enddefine;

if systrmdev(popdevin) then test() endif;


/* --- Revision History ---------------------------------------------------
--- Aaron Sloman, Oct 14 1996
	Revised to use "!" with patterns
--- John Gibson, Aug 24 1993
		Uses showtree instead of new_sh*owtree
 */
