/* --- Copyright University of Birmingham 1997. All rights reserved. ------
 > File:            $poplocal/local/lib/hangman.p
 > Purpose:			Revised version of LIB HANGMAN, for vturtle.
 > Author:          Aaron Sloman, Dec 26 1996 (see revisions)
 > Documentation:	Below
 > Related Files:
 */


/*  --- Copyright University of Sussex 1986.  All rights reserved. ---------
 >  File:           $usepop/master/C.all/lib/lib/hangman.p
 >  Purpose:        Hangman spelling game. Works only on Visual 200 terminal.
 >  Author:         Aaron Sloman, 1980 (see revisions)
 >  Related Files:  LIB *V200GRAPH  LIB *ACTIVE $usepop/pop/lib/demo/mkhang
 */

/*

To play, give the command

	play();

Then decide whether you want words of no more than four letters, and if so
type "y" in answer to the question, otherwise "n".

The computer will choose a word for you to guess.

Keep guessing by typing letters. You will be shown which letters you
have already guessed, and which are correct and if so where they occur
in the word.

Each time you guess a letter that is not in the word a part of a picture
will be drawn: the picture shows a gallows and a man hanging from
it. The following are the items that are drawn in turn:
	Gallows Base, gallows upright, gallows arm,
	Man's head, man's neck, left limb, right limb,
	man's trunk, man's left leg, right leg.

When all have been drawn the game is over. You'll be asked whether
you would like to try to guess the word again. If you answer "n" you'll
be asked whether you want to see the word.


*/

section;

uses vturtle

global vars words;	;;; defined below.

lvars
	wonstring = '', Xwonstring = 0, Ywonstring = 0,
	Xman,Yman,Xword,Yword,Xletters,Yletters,
	me,you;

18 -> Xword; 5 -> Yword;
18 -> Xletters; 10 -> Yletters;
18 -> Xwonstring; 3 ->  Ywonstring;

poppid + systime(); -> ranseed;

define insert(inn,letters);
	if letters == [] then [^inn]
	elseif inn(1) < hd(letters)(1)
	then    inn :: letters
	else    hd(letters)::insert(inn,tl(letters))
	endif
enddefine;

define plantstring(x,y,string);
	jumpto(x,y); 0 -> heading;
	vedcleartail();
	vedinsertstring(string);
	jumpto(x + datalength(string) -1 , y);
enddefine;

define Base();
	"=" -> paint;
	jumpto(1,1); 0 -> heading;
	draw(11);
enddefine;

define Upright();
		jumpto(5,1);drawto(5,15);
enddefine;

define Arm();
		jumpto(5,15); 0 -> heading; draw(5); turn(-90);
		"|" -> paint;
		draw(2);
		xposition -> Xman; yposition -> Yman;
enddefine;

define Head();
		"*" -> paint;
		jumpto(Xman,Yman);
		-45 -> heading;
		repeat 4 times tdraw(2);turn(-90) endrepeat;
		-90 -> heading; jump(3);
		xposition -> Xman; yposition -> Yman;
enddefine;

define startdown(ang);
		jumpto(Xman,Yman);
		ang -> heading;
enddefine;

define Neck();
		startdown(-90);
		draw(1);
		xposition -> Xman; yposition ->Yman;
enddefine;

define limb(ang);
		startdown(ang); draw(6)
enddefine;

define Llimb();
		limb(-135);
enddefine;

define Rlimb();
		limb(-45);
enddefine;

define Trunk();
		startdown(-90);
		draw(3);
		xposition -> Xman; yposition -> Yman;
enddefine;

lvars hangman;
[%Base,Upright,Arm,Head,Neck,Llimb,Rlimb,Trunk,Llimb,Rlimb%] -> hangman;

define prepare_picture(word);
	dlocal paint;
	vededit(popturtlefile, vedhelpdefaults);
	ved_q();	;;; get rid of any existing picture
	vededit(popturtlefile, vedhelpdefaults);
	newpicture(Xletters+25,12);
	;;; turtle();
	"-" -> paint;
	jumpto(1, 16);
	drawto(Xletters+25, 16);

	plantstring(Xletters,Yletters+1,'Guesses:');
	plantstring(Xword,Yword+1,'The word:');
	jumpto(Xword,Yword);
	0 -> heading;
	"_" -> paint;
	draw(length(word) - 1);
	plantstring(Xwonstring, Ywonstring, wonstring);
	rawoutflush();
	pr('The word has ' >< length(word) >< ' letters.\
	When you guess a correct letter, its position will be shown\
	Otherwise the picture of a man on gallows will be extended.\
	Try to guess all the letters before the man is finished.\
	To redraw display press CTRL L\n');
enddefine;

/*
prepare_picture('catch');
vededit('picture', vedhelpdefaults),
ved_q(),
newpicture(10,20);
draw(50);
*/

define getchar(mess)->char;
	vars x;
	0 -> char;
	0 -> x;
	until islowercode(char) do
		x + 1 -> x;
		pr(newline);
		vedcheck();
		pr(mess>< '? ');
		pr(newline);
		vedcheck();
		rawcharin()-> char;
		vedcheck();
		;;; give up if more than 30 non-letters are typed.
		if x > 30 or char == termin then interrupt() endif;
		if char == `\^V` ;;; ctrl V
		or char == `\^L`  ;;; CTRL L
		then
			display()
		endif;
		if isuppercode(char)
		then uppertolower(char) -> char
		endif;
		rawcharout(char);
		vedcheck();
	enduntil;
enddefine;

define answeryes(mess);
	 pr(newline);
	 getchar(mess >< '(y/n)') /== `n`
enddefine;



define checkout(letters,inn) -> x;
	0 -> x;
	jumpto(Xword,Yword);
	until letters == [] do
		if inn == hd(letters)
		then inn -> picture(xposition,yposition); 1 + x-> x
		endif;
		jump(1); tl(letters) -> letters
	enduntil
enddefine;

define getguesses(letters)->won;
	dlocal hangman;
	lvars inn, x, count = 0,
		tried = [], won = false;
	until hangman == [] or won
	do
		getchar('Type a letter') -> inn;
		consword(inn,1) -> inn;
		if   member(inn, tried)
		then    pr('\nyou\'ve tried that before. Try another\n')
		else
			insert(inn,tried) -> tried;
			jumpto(Xletters,Yletters); 0 -> heading;
			applist(tried, procedure x;
					x-> picture(xposition,yposition);
					jump(1); endprocedure);
			checkout(letters,inn) -> x;
			if x > 0 then
				x + count -> count;
				count == length(letters) -> won
			else apply(dest(hangman) -> hangman);
			endif;
		endif
	enduntil
enddefine;

define play();
	dlocal cucharout;

	shuffle(words) -> words;
	define prmishap;
		vars cucharout;
		charout -> cucharout;
		sysprmishap();
		pr('\n\nTYPE CTRL-C to CONTINUE'); charin();
	enddefine;

	;;; prevent charout messing up format.
	false -> poplinewidth;
	false -> poplinemax;

	define interrupt;
		pr('\nBye\n');
		exitfrom(play);
	enddefine;

	0 ->>me->you;
	true-> playagain;
	'Me: 0  You: 0' -> wonstring;
	pr('Please answer questions by typing only "y" or "n"\n');
	if answeryes('Would you like to have only short words')
	then    maplist(words,procedure x; if length(x) < 5 then x endif end)
			->words
	endif;
	until words == [] or
		(not(playagain) and not(answeryes('Play some more')))
	do
		false -> playagain;
		word_string(hd(words)) -> word;
		prepare_picture(word);
		getguesses(unpackitem(word)) -> won;
		pr('\n' ><
			if won then 1+you -> you; '\nWELL DONE!!\n'
			else 1+me -> me; '\nBAD LUCK\n' endif);

		'Me: ' >< me >< '   You: ' >< you -> wonstring;
		plantstring(Xwonstring,Ywonstring,wonstring);
		if not(won)
				and answeryes('Try that word again')
		then    true -> playagain;
		elseif won or answeryes('Want to see the word')
		then    pr('\nIt was "' >< word >< '"\n');
			tl(words) -> words;
		else    tl(words) -> words
		endif;
	enduntil;
	pr(wonstring >< '\n\nBye\n');
enddefine;



[
 accidental ably absent although alligator angry antelope
 apple applicability an angle ape arch ardent
 arduous argue at ate
 aunt average awe awkward awsome aye
 baby bad banana bard because bed beard bending better
 bind bird blanket blind blue bridge bright builder
 bunk busy by
 call cat catch chair ceiling century charge chattering
 chest child children chink chip clasp clanger
 come cope computer
 confident cordial corporal cost crash creak
 curly cup cyst
 dancing dairy damage dawn day dazzle
 deer define deride devote diagnose diarrhoeia
 dig diminutive dint dire dirty do doctor dog dome
 door down dowry drain drizzle dull dumbfounded dump
 eating easy ecstatic edge electricity elephant elevate
 elk else embarrassed embezzler ensue entire
 enthusiastic estimated even eventually every example excellent
 expectation expeditious extent extinct extra
 fade fancy fast fed feel festive few
 fiddle fidget fill finger fish fist fistful flamboyant
 flew flay flute fly fool fragment fray fruit fuel
 fungus funny furnishings
 gate garrison gauge gaudy gay generous gent gentle genuine
 gladly glanced go gone good grandiose grumpy
 guard guarantee guava gun guru gutter guzzle
 hairy hand happier hatch headache heart hearty
 hellish help here herewith hop hope hopping
 hosier hundred hunting hushed hurtle hymns
 ice idiot idolatry if ignore imbue impropriety impune inn
 incomparable indolent indulge inflation ink insolent irrigate
 irritate insidious itchy
 jangle jarring jaw jealousy jester joke joker jump
 judge juicy justify
 kangaroo keenly keep kidney kindle kin kindly king
 kitchen knapsack knight know knowledge knuckle
 lady ladylike lazy languid lard lavish leader lend
 leverage lewd light like limped link liquefy
 liquid load loathsome lobster lord lucid lucky
 lump lurch lusty
 machine mad magnificent marauding marriage meat medicine
 meet mew milk mince misinterpret mislead misled misty
 munch music musty my
 nail name narcissistic nasty naval navel navy
 nefarious never new nicest nine ninety none
 nonentity noon nose
 nostril notary nought novel nozzle now nowadays
 numb nut
 obelisk odour often omen ominous one onerous
 ontology opal optical orange ordinary
 ostensibly ostentatious ousted our out outlaw
 outrageous oven over owl owlish oxygen
 oyster
 painted pardon passion pastel pauper paw
 pea peal peasant peat pending pensive pensively
 persist pet petal pew pheasant philosophy
 picture picturesque pile pinch pistol pitcher
 placid plagiarism plan plant please pleasantry plenty plentiful
 ply poke polish
 pony pompously portray prairie prank prawn
 prejudice pretty prisoner probe
 professional programme pry
 quaint quality quantity quarrel quaver quay
 queen querulous query quickly quiet quintuplet
 quirk quiver
 quota quorum quote
 rabbit radiated raft rake rally rancid
 raucous really reason reed relieve rendesvous rent
 respiration riddle rift rival rocking rosary
 rude rue ruffian ruler run rusty
 sadly saintly sardine savage saw saxophone
 seal sentimental set shake shrunken simpleton six skip
 skullduggery sky snippet snowy socks
 soil stank stop stupidly switch
 tangle take teacher terminate toe top tortoise trap tree
 triangle try
 udder ugly uglier umbilical umbrella unambitious uncle unctuous
 under underneath unhinged unenthusiastic uniform unique up
 upper upstairs use useful usually usury utility utopian
 vacuum vague vault valley value vampire vast vehicle veils vendor
 venerable vent
 vented vermin verse versify ventriloquist very vested
 vestibule veterinary vicious viewer vigorously vile vixen
 void volley voltage voluptuous voracious vortex vow voyage vulpine
 waddle waft wage wager waif wail waist wait
 walking wake waking wall walled wander wane wanted
 ward wart warranted wash washing waste water wave wavy way
 weapon weary weave weird went wet whale whatever when whenever
 where whey while whirl whirring whispery who whole
 wind windy wink winter wintry
 wire wiry wisdom wisely wish wishing wistful
 wonder wooden worldly worse worsen worst woven
 yacht yank yard yarn yaw yawn yawning
 year yearly yearn yeast yellow yellowish yelp yes yesterday
 yet yolk yonder yore yoyo you young your yours youth youthful
 zeal zealous zebra zero zip zoo zoom zooming zoological zoology zombie
] -> words;

/*
if systrmdev(popdevin) then
	play();
endif;
*/
pr('\n\nTo test your spelling please type \n\n\tplay();\n');

endsection;

/*  --- Revision History ---------------------------------------------------
--- Aaron Sloman, Dec 10 1997
	fixed some declarations.

--- Aaron Sloman, Dec 26 1996
	Finally fixed to work properly with LIB VTURTLE. Added several
	more words.
--- Aaron Sloman, Aug 18 1986 time for new words
 */
